@@ -431,6 +431,30 @@ public class PGPPublicKey
         
         return null;
     }
+
+    /**
+     * Return any signatures associated with the passed in key identifier keyID.
+     *
+     * @param keyID the key id to be matched.
+     * @return an iterator of PGPSignature objects issued by the key with keyID.
+     */
+    public Iterator getSignaturesForKeyID(
+        long   keyID)
+    {
+        List sigs = new ArrayList();
+
+        for (Iterator it = getSignatures(); it.hasNext();)
+        {
+            PGPSignature sig = (PGPSignature)it.next();
+
+            if (sig.getKeyID() == keyID)
+            {
+                sigs.add(sig);
+            }
+        }
+
+        return sigs.iterator();
+    }
     
     /**
      * Return an iterator of signatures associated with the passed in user attributes.
@@ -503,6 +527,26 @@ public class PGPPublicKey
         }
     }
 
+    /**
+     * Return all signatures/certifications directly associated with this key (ie, not to a user id).
+     *
+     * @return an iterator (possibly empty) with all signatures/certifications.
+     */
+    public Iterator getKeySignatures()
+    {
+        if (subSigs == null)
+        {
+            List sigs = new ArrayList();
+            sigs.addAll(keySigs);
+            
+            return sigs.iterator();
+        }
+        else
+        {
+            return subSigs.iterator();
+        }
+    }
+
     public PublicKeyPacket getPublicKeyPacket()
     {
         return publicPk;
