@@ -9,10 +9,12 @@ import java.util.Date;
 import java.util.Iterator;
 import java.util.List;
 
+import org.bouncycastle.asn1.x9.ECNamedCurveTable;
 import org.bouncycastle.bcpg.BCPGKey;
 import org.bouncycastle.bcpg.BCPGOutputStream;
 import org.bouncycastle.bcpg.ContainedPacket;
 import org.bouncycastle.bcpg.DSAPublicBCPGKey;
+import org.bouncycastle.bcpg.ECPublicBCPGKey;
 import org.bouncycastle.bcpg.ElGamalPublicBCPGKey;
 import org.bouncycastle.bcpg.PublicKeyAlgorithmTags;
 import org.bouncycastle.bcpg.PublicKeyPacket;
@@ -81,6 +83,10 @@ public class PGPPublicKey
             {
                 this.keyStrength = ((ElGamalPublicBCPGKey)key).getP().bitLength();
             }
+            else if (key instanceof ECPublicBCPGKey)
+            {
+                this.keyStrength = ECNamedCurveTable.getByOID(((ECPublicBCPGKey)key).getCurveOID()).getCurve().getFieldSize();
+            }
         }
     }
 
@@ -204,12 +210,23 @@ public class PGPPublicKey
     /**
      * @return number of valid days from creation time - zero means no
      * expiry.
+     * @deprecated use getValidSeconds(): greater than version 3 keys may be valid for less than a day.
      */
     public int getValidDays()
     {
         if (publicPk.getVersion() > 3)
         {
-            return (int)(this.getValidSeconds() / (24 * 60 * 60));
+            long delta = this.getValidSeconds() % (24 * 60 * 60);
+            int days = (int)(this.getValidSeconds() / (24 * 60 * 60));
+
+            if (delta > 0 && days == 0)
+            {
+                return 1;
+            }
+            else
+            {
+                return days;
+            }
         }
         else
         {
@@ -439,7 +456,7 @@ public class PGPPublicKey
      * @param id the id to be matched.
      * @return an iterator of PGPSignature objects.
      */
-    public Iterator getSignaturesForID(
+    public Iterator<PGPSignature> getSignaturesForID(
         String   id)
     {
         return getSignaturesForID(new UserIDPacket(id));
@@ -451,12 +468,36 @@ public class PGPPublicKey
      * @param rawID the id to be matched in raw byte form.
      * @return an iterator of PGPSignature objects.
      */
-    public Iterator getSignaturesForID(
+    public Iterator<PGPSignature> getSignaturesForID(
         byte[]   rawID)
     {
         return getSignaturesForID(new UserIDPacket(rawID));
     }
 
+    /**
+     * Return any signatures associated with the passed in key identifier keyID.
+     *
+     * @param keyID the key id to be matched.
+     * @return an iterator of PGPSignature objects issued by the key with keyID.
+     */
+    public Iterator<PGPSignature> getSignaturesForKeyID(
+        long   keyID)
+    {
+        List sigs = new ArrayList();
+
+        for (Iterator it = getSignatures(); it.hasNext();)
+        {
+            PGPSignature sig = (PGPSignature)it.next();
+
+            if (sig.getKeyID() == keyID)
+            {
+                sigs.add(sig);
+            }
+        }
+
+        return sigs.iterator();
+    }
+
     private Iterator getSignaturesForID(
         UserIDPacket   id)
     {
@@ -542,6 +583,26 @@ public class PGPPublicKey
         }
     }
 
+    /**
+     * Return all signatures/certifications directly associated with this key (ie, not to a user id).
+     *
+     * @return an iterator (possibly empty) with all signatures/certifications.
+     */
+    public Iterator getKeySignatures()
+    {
+        if (subSigs == null)
+        {
+            List sigs = new ArrayList();
+            sigs.addAll(keySigs);
+            
+            return sigs.iterator();
+        }
+        else
+        {
+            return subSigs.iterator();
+        }
+    }
+
     public PublicKeyPacket getPublicKeyPacket()
     {
         return publicPk;
@@ -620,13 +681,24 @@ public class PGPPublicKey
             }
         }
     }
-    
+
+    /**
+     * Check whether this (sub)key has a revocation signature on it.
+     *
+     * @return boolean indicating whether this (sub)key has been revoked.
+     * @deprecated this method is poorly named, use hasRevocation().
+     */
+    public boolean isRevoked()
+    {
+        return hasRevocation();
+    }
+
     /**
      * Check whether this (sub)key has a revocation signature on it.
      * 
-     * @return boolean indicating whether this (sub)key has been revoked.
+     * @return boolean indicating whether this (sub)key has had a (possibly invalid) revocation attached..
      */
-    public boolean isRevoked()
+    public boolean hasRevocation()
     {
         int ns = 0;
         boolean revoked = false;
