@@ -2,6 +2,7 @@ package org.bouncycastle.pqc.jcajce.provider.test;
 
 import java.security.Security;
 
+import junit.extensions.TestSetup;
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
@@ -23,13 +24,35 @@ public class AllTests
         {
             Security.addProvider(new BouncyCastlePQCProvider());
         }
-        
+
+        suite.addTestSuite(Sphincs256Test.class);
         suite.addTestSuite(RainbowSignatureTest.class);
         suite.addTestSuite(McElieceFujisakiCipherTest.class);
         suite.addTestSuite(McElieceKobaraImaiCipherTest.class);
         suite.addTestSuite(McEliecePointchevalCipherTest.class);
-        suite.addTestSuite(McEliecePKCSCipherTest.class);
+        suite.addTestSuite(McElieceCipherTest.class);
+        suite.addTestSuite(McElieceKeyPairGeneratorTest.class);
+        suite.addTestSuite(McElieceCCA2KeyPairGeneratorTest.class);
 
-        return suite;
+        return new BCTestSetup(suite);
+    }
+
+    static class BCTestSetup
+        extends TestSetup
+    {
+        public BCTestSetup(Test test)
+        {
+            super(test);
+        }
+
+        protected void setUp()
+        {
+            Security.addProvider(new BouncyCastlePQCProvider());
+        }
+
+        protected void tearDown()
+        {
+            Security.removeProvider("BCPQC");
+        }
     }
 }
